
#include "vpostproc/postproc.hh"


VideoImageBuffer VideoPostprocessor::s_bufprov;


VideoPostprocessor::VideoPostprocessor()
  : d_next(NULL),
    nReplacements(0)
{
  replacement = new Replacement[MaxReplacements];
}

VideoPostprocessor::~VideoPostprocessor()
{
  Assert(nReplacements==0);
  delete[] replacement;
}

DecodedImageData* VideoPostprocessor::GetOutputBuffer(ImageSpec_YUV spec,
						      uint3 picture_type,bool maymodify)
{
  Assert(d_next);

  if (!maymodify)
    {
      Assert(nReplacements<MaxReplacements);
      replacement[nReplacements].replacement = s_bufprov.GetOutputBuffer(spec,picture_type,false);
      replacement[nReplacements].spec = spec;
      replacement[nReplacements].pic_type = picture_type;

      return replacement[nReplacements++].replacement;
    }
  else
    return d_next->GetOutputBuffer(spec,picture_type,true);
}


static void CopyImageData(const Image_YUV<Pixel>& src,Image_YUV<Pixel>& dst)
{
  ImageParam_YUV param;
  src.GetParam(param);

  Pixel*const* dyp = dst.AskFrameY();
  Pixel*const* dup = dst.AskFrameU();
  Pixel*const* dvp = dst.AskFrameV();

  const Pixel*const* syp = src.AskFrameY_const();
  const Pixel*const* sup = src.AskFrameU_const();
  const Pixel*const* svp = src.AskFrameV_const();

  for (int y=0;y<param.height;y++)
    for (int x=0;x<param.width;x++)
      {
	dyp[y][x] = syp[y][x];
      }

  int w,h;
  param.GetChromaSizes(w,h);

  for (int y=0;y<h;y++)
    for (int x=0;x<w;x++)
      {
	dup[y][x] = sup[y][x];
	dvp[y][x] = svp[y][x];
      }
}


void VideoPostprocessor::ShowOutputBuffer(DecodedImageData* dimg)
{
  for (int i=0;i<nReplacements;i++)
    {
      if (replacement[i].replacement==dimg)
	{
	  DecodedImageData* outputimg = d_next->GetOutputBuffer(replacement[i].spec,
								replacement[i].pic_type,true);

	  CopyImageData(dimg->m_image,outputimg->m_image);
	  outputimg->m_picdata1 = dimg->m_picdata1;
	  outputimg->m_picdata2 = dimg->m_picdata2;
	  outputimg->m_timing   = dimg->m_timing;

	  Render(outputimg);
	  Assert(d_next);
	  d_next->ShowOutputBuffer(outputimg);
	  outputimg->m_picdata1 = outputimg->m_picdata2 = NULL;
	  d_next->FreeOutputBuffer(outputimg);

	  return;
	}
    }

  Render(dimg);
  Assert(d_next);
  d_next->ShowOutputBuffer(dimg);
}


void              VideoPostprocessor::FreeOutputBuffer(DecodedImageData* dimg)
{
  for (int i=0;i<nReplacements;i++)
    {
      if (replacement[i].replacement==dimg)
	{
	  Assert(d_next);
	  s_bufprov.FreeOutputBuffer(replacement[i].replacement);

	  replacement[i] = replacement[nReplacements-1];
	  nReplacements--;
	  return;
	}
    }

  Assert(d_next);
  d_next->FreeOutputBuffer(dimg);
}

